"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzureFunctionsController = void 0;
require("reflect-metadata");
const helpers_1 = require("./helpers");
class AzureFunctionsController {
    /**
     * @param authService The AuthService to use with the '@CheckAuth' decorater
     */
    constructor(authService) {
        if (authService)
            this.authService = authService;
    }
    /**
     * Custom setup method that is ran first before validation.
     *
     * @param context AzureFunction context object
     * @param req HTTPRequest object
     * @returns An error FunctionResponse or null if no error
    */
    preValidation(context, req) {
        return __awaiter(this, void 0, void 0, function* () { return null; });
    }
    /**
     * Custom validation method that is ran before the GET/POST/PATCH logic
     *
     * @param context AzureFunction context object
     * @param req HTTPRequest object
     * @returns An error FunctionResponse or null if no error
     */
    validation(context, req) {
        return __awaiter(this, void 0, void 0, function* () { return null; });
    }
    /**
     * Custom setup method that is ran after validation but before the GET/POST/PATCH logic
     *
     * @param context AzureFunction context object
     * @param req HTTPRequest object
     * @returns An error FunctionResponse or null if no error
    */
    postValidation(context, req) {
        return __awaiter(this, void 0, void 0, function* () { return null; });
    }
    /**
     * Builds the Azure Function call this and export the returned function.
     *
     * @returns An AzureFunction
     */
    build() {
        let routes = {
            'GET': {},
            'POST': {},
            'PATCH': {},
            'DELETE': {},
            'CONNECT': {},
            'OPTIONS': {},
            'PUT': {},
            'TRACE': {},
            'HEAD': {}
        };
        return (context, req) => __awaiter(this, void 0, void 0, function* () {
            // Use relection to get a list of all the methods the user has implemented
            const proto = Reflect.getPrototypeOf(this);
            const methods = Object.getOwnPropertyNames(proto).filter(m => m !== 'constructor');
            // For each of those check if the @Endpoint decorater was used by checking the reflected metadata
            for (let i = 0; i < methods.length; i++) {
                const methodName = methods[i];
                const httpMethod = Reflect.getMetadata('tn:anotations:method', this, methodName);
                const httpRoute = Reflect.getMetadata('tn:anotations:route', this, methodName);
                if (httpMethod && httpRoute)
                    routes[httpMethod][httpRoute] = proto[methodName].bind(this);
            }
            // Split on /api/ and take the later half then split on ? to seperate the query params out.
            const route = req.url.split('/api/')[1].split('?')[0];
            const method = req.method;
            const endpoint = routes[method][route];
            if (!endpoint)
                return context.res = (0, helpers_1.responseFactory)(404, `${method}: ${route} could not be resolved`);
            // Run the preValidation, validation and postValidation functions
            let rval;
            if (rval = yield this.preValidation(context, req))
                return context.res = rval;
            if (rval = yield this.validation(context, req))
                return context.res = rval;
            if (rval = yield this.postValidation(context, req))
                return context.res = rval;
            // Finally run the function that matches the method & route
            return context.res = yield endpoint(context, req);
        });
    }
}
exports.AzureFunctionsController = AzureFunctionsController;
//# sourceMappingURL=index.js.map